@@ -29,7 +29,8 @@ Input, recurrent network, and output layer.
 
 __author__ = "Richard Gast"
 __status__ = "Development"
-__version__ = "0.1.5"
+__version__ = "0.3.0"
 
 from .network import Network
-from .utility import random_connectivity, circular_connectivity
+from .utility import random_connectivity, circular_connectivity, input_connections
+from .utility import wta_score, readout
