@@ -34,6 +34,13 @@ class Network:
         self._var_map = var_map if var_map else {}
         self._model = None
 
+    def __getitem__(self, item: int):
+        try:
+            return self._model[item]
+        except AttributeError:
+            self.compile()
+        return self._model[item]
+
     @classmethod
     def from_yaml(cls, node: Union[str, NodeTemplate], weights: np.ndarray, source_var: str, target_var: str,
                   input_var_ext: str, output_var: str, input_var_net: str = None, spike_var: str = None, op: str = None,
@@ -114,7 +121,7 @@ class Network:
         """
         return self._model
 
-    def add_input_layer(self, n: int, m: int, weights: torch.Tensor = None, trainable: bool = False,
+    def add_input_layer(self, n: int, m: int, weights: np.ndarray = None, trainable: bool = False,
                         dtype: torch.dtype = torch.float64) -> InputLayer:
         """Add an input layer to the network. Networks can have either 1 or 0 input layers.
 
@@ -147,7 +154,7 @@ class Network:
         # return layer
         return self.input_layer
 
-    def add_output_layer(self, n: int, k: int, weights: torch.Tensor = None, trainable: bool = False,
+    def add_output_layer(self, n: int, k: int, weights: np.ndarray = None, trainable: bool = False,
                          activation_function: str = None, dtype: torch.dtype = torch.float64) -> OutputLayer:
         """Add an output layer to the network. Networks can have either 1 or 0 output layers.
 
@@ -195,7 +202,7 @@ class Network:
 
     def train(self, inputs: np.ndarray, targets: np.ndarray, optimizer: str = 'sgd', optimizer_kwargs: dict = None,
               loss: str = 'mse', loss_kwargs: dict = None, lr: float = 1e-3, device: str = None,
-              sampling_steps: int = 100, verbose: bool = True, **kwargs) -> Observer:
+              sampling_steps: int = 100, optimizer_steps: int = 1, verbose: bool = True, **kwargs) -> Observer:
         """Optimize model parameters such that the model output matches the provided targets as close as possible.
 
         Parameters
@@ -210,9 +217,11 @@ class Network:
             Name of the optimization algorithm to use. Available options are:
             - 'sgd' for `torch.optim.SGD`
             - 'adam' for `torch.optim.Adam`
+            - 'adamw' for torch.optim.AdamW
             - 'adagrad' for `torch.optim.Adagrad`
-            - 'lbfgs' for `torch.optim.LBFGS`
+            - 'adadelta' for `torch.optim.Adadelta`
             - 'rmsprop' for `torch.optim.RMSprop`
+            - 'rprop' for `torch.optim.Rprop`
         optimizer_kwargs
             Additional keyword arguments provided to the initialization of the optimizer.
         loss
@@ -249,7 +258,7 @@ class Network:
         inp_tensor = torch.tensor(inputs)
         target_tensor = torch.tensor(targets)
         if inp_tensor.shape[0] != target_tensor.shape[0]:
-            raise ValueError('Wrong dimensions of input and target output. Please make shure that `inputs` and '
+            raise ValueError('Wrong dimensions of input and target output. Please make sure that `inputs` and '
                              '`targets` agree in the first dimension.')
 
         # set up model
@@ -275,27 +284,13 @@ class Network:
         # optimization
         ##############
 
-        steps = inp_tensor.shape[0]
         t0 = perf_counter()
-        for step in range(steps):
-
-            # forward pass
-            prediction = model(inp_tensor[step, :])
-
-            # loss calculation
-            error = loss(prediction, target_tensor[step, :])
-
-            # error backpropagation
-            optimizer.zero_grad()
-            error.backward(**error_kwargs)
-            optimizer.step(**step_kwargs)
-
-            # results storage
-            if step % sampling_steps == 0:
-                if verbose:
-                    print(f'Progress: {step}/{steps} training steps finished.')
-                obs.record(prediction, error.item(), self.rnn_layer.record(rec_vars))
-
+        if len(inp_tensor.shape) > 2:
+            self._train_epochs(inp_tensor, target_tensor, model, loss, optimizer, obs, rec_vars, error_kwargs,
+                               step_kwargs, sampling_steps=sampling_steps, optim_steps=optimizer_steps, verbose=verbose)
+        else:
+            self._train(inp_tensor, target_tensor, model, loss, optimizer, obs, rec_vars, error_kwargs, step_kwargs,
+                        sampling_steps=sampling_steps, optim_steps=optimizer_steps, verbose=verbose)
         t1 = perf_counter()
         print(f'Finished optimization after {t1-t0} s.')
         return obs
@@ -338,9 +333,6 @@ class Network:
         # transform inputs into tensors
         inp_tensor = torch.tensor(inputs)
         target_tensor = torch.tensor(targets)
-        if inp_tensor.shape[0] != target_tensor.shape[0]:
-            raise ValueError('Wrong dimensions of input and target output. Please make shure that `inputs` and '
-                             '`targets` agree in the first dimension.')
 
         # set up model
         model = self.compile(device) if self._model is None else self._model
@@ -349,7 +341,7 @@ class Network:
         loss = self._get_loss_function(loss, loss_kwargs=loss_kwargs)
 
         # initialize observer
-        obs_kwargs = retrieve_from_dict(['record_output', 'record_loss', 'record_vars'], **kwargs)
+        obs_kwargs = retrieve_from_dict(['record_output', 'record_loss', 'record_vars'], kwargs)
         obs = Observer(dt=self.rnn_layer.dt, **obs_kwargs)
         rec_vars = [self._relabel_var(v) for v in obs.recorded_rnn_variables]
 
@@ -422,7 +414,7 @@ class Network:
 
         return obs
 
-    def compile(self, device: Union[str, None]) -> Sequential:
+    def compile(self, device: str = None) -> Sequential:
         """Connects the `InputLayer`, `RNNLayer` and `OutputLayer` of this model via a `torch.nn.Sequential` instance.
         Input and output layers are optional.
 
@@ -445,6 +437,73 @@ class Network:
             model.to(device)
         return model
 
+    def _train(self, inp: torch.Tensor, target: torch.Tensor, model: Sequential, loss: Callable,
+               optimizer: torch.optim.Optimizer, obs: Observer, rec_vars: list, error_kwargs: dict, step_kwargs: dict,
+               sampling_steps: int = 100, optim_steps: int = 1, verbose: bool = False):
+
+        if inp.shape[0] != target.shape[0]:
+            raise ValueError('Wrong dimensions of input and target output. Please make sure that `inputs` and '
+                             '`targets` agree in the first dimension.')
+
+        steps = inp.shape[0]
+        for step in range(steps):
+
+            # forward pass
+            prediction = model(inp[step, :])
+
+            # loss calculation
+            error = loss(prediction, target[step, :])
+
+            # error backpropagation
+            error.backward(**error_kwargs)
+            if step % optim_steps == 0:
+                optimizer.step(**step_kwargs)
+                optimizer.zero_grad()
+
+            # results storage
+            if step % sampling_steps == 0:
+                if verbose:
+                    print(f'Progress: {step}/{steps} training steps finished.')
+                obs.record(prediction, error.item(), self.rnn_layer.record(rec_vars))
+
+    def _train_epochs(self, inp: torch.Tensor, target: torch.Tensor, model: Sequential, loss: Callable,
+                      optimizer: torch.optim.Optimizer, obs: Observer, rec_vars: list, error_kwargs: dict,
+                      step_kwargs: dict, sampling_steps: int = 100, optim_steps: int = 1, verbose: bool = False):
+
+        if inp.shape[0] != target.shape[0] or inp.shape[1] != target.shape[1]:
+            raise ValueError('Wrong dimensions of input and target output. Please make sure that `inputs` and '
+                             '`targets` agree in the first dimension (epochs) and second dimension (steps per epoch).')
+        epochs = inp.shape[0]
+        steps = inp.shape[1]
+        for epoch in range(epochs):
+
+            # go through steps of the epoch
+            epoch_loss = 0
+            for step in range(steps):
+
+                # forward pass
+                prediction = model(inp[epoch, step, :])
+
+                # loss calculation
+                error = loss(prediction, target[epoch, step, :])
+                epoch_loss += error.item()
+
+                # error backpropagation
+                error.backward(**error_kwargs)
+                if step % optim_steps == 0:
+                    optimizer.step(**step_kwargs)
+                    optimizer.zero_grad()
+
+                # results storage
+                if step % sampling_steps == 0:
+                    obs.record(prediction, error.item(), self.rnn_layer.record(rec_vars))
+
+            # display progress
+            if verbose:
+                print(f'Progress: {epoch+1}/{epochs} training epochs finished.')
+                print(f'Epoch loss: {epoch_loss}.')
+                print('')
+
     def _relabel_var(self, var: str) -> str:
         try:
             return self._var_map[var]
@@ -462,14 +521,18 @@ class Network:
             opt = torch.optim.SGD
         elif optimizer == 'adam':
             opt = torch.optim.Adam
+        elif optimizer == 'adamw':
+            opt = torch.optim.AdamW
         elif optimizer == 'adagrad':
             opt = torch.optim.Adagrad
-        elif optimizer == 'lbfgs':
-            opt = torch.optim.LBFGS
+        elif optimizer == 'adadelta':
+            opt = torch.optim.Adadelta
         elif optimizer == 'rmsprop':
             opt = torch.optim.RMSprop
+        elif optimizer == 'rprop':
+            opt = torch.optim.Rprop
         else:
-            raise ValueError('Invalid optimizer choice. Please see the documentation of the `Network.run()` '
+            raise ValueError('Invalid optimizer choice. Please see the documentation of the `Network.train()` '
                              'method for valid options.')
         return opt(model_params, lr=lr, **optimizer_kwargs)
 
@@ -498,7 +561,7 @@ class Network:
             from torch.nn import HingeEmbeddingLoss
             l = HingeEmbeddingLoss
         else:
-            raise ValueError('Invalid loss function choice. Please see the documentation of the `Network.run()` '
+            raise ValueError('Invalid loss function choice. Please see the documentation of the `Network.train()` '
                              'method for valid options.')
         return l(**loss_kwargs)
 
